local menu = {}
menu.__index = menu
local lg = love.graphics
local tinsert = table.insert
local floor = math.floor





local state = {}
state.disabled = {30,30,70,255}
state.enabled =  {70,70,100,255}
state.material =   {200,200,230,255}
state.available = {130,130,160,255}

function menu:CheckTileLink(x,y)
	local tile = self.grid[x] and self.grid[x][y]

	if tile and tile.linked and tile.material and (not tile.checked) then
		tile.checked = true
		for i = -1, 1 do
			for j = -1, 1 do
				if i == 0 or j == 0 then 
					local neighborTile = self.grid[x+i] and self.grid[x+i][y+j]
					if neighborTile then
						if neighborTile.material then
							neighborTile.linked = true
							self:CheckTileLink(x+i,y+j)
						end
					end
				end
			end
		end
	end
end



function menu:ClickTile(tile)

	if tile.picker and self.gems > 0 then
		if tile.material then
			self.gems = self.gems + 1
			tile.material = nil
		else
			self.gems = self.gems - 1
			tile.material = self.currentMaterial
		end
	end
end

function menu:GetSeedX()
	return self.template.seedX
end

function menu:GetSeedY()
	return self.template.seedY
end

function menu:GetWidth()
	return self.template.config.width
end

function menu:GetHeight()
	return self.template.config.height
end

function menu:TileIterator(func)
	for i = 1, self:GetWidth() do
		for j = 1, self:GetHeight() do
			func(self,i,j)
		end
	end
end

function ResetTileLinks(self,i,j)
	local tile = self.grid[i][j]
	tile.linked = false
	tile.available = false
	tile.checked = false
end

function RemoveUnlinkedTile(self,i,j)
	local tile = self.grid[i][j]
	if tile.linked == false then
		if tile.material then
			self.gems = self.gems + 1
			tile.material = false
		end
	end
end

function menu:GetTileSize()
	return self.template.config.tileSize
end


function DrawTile(self,i,j)
	local tile = self.grid[i][j]
	if tile.picker then
		lg.setColor(unpack(tile.picker.color))
		local tileSize = self:GetTileSize()
		lg.rectangle("fill", i*tileSize,j*tileSize,tileSize,tileSize)
		lg.rectangle("line", i*tileSize,j*tileSize,tileSize,tileSize)
		
		if tile.material then
			lg.setColor(tile.material.color)
			lg.rectangle("fill", i*tileSize+1,j*tileSize+1,tileSize-2,tileSize-2)
		end
		lg.setColor(255,255,255,255)
	end
end


function UpdateTileAvailability(self,x,y)
	local tile = self.grid[x][y]

	for i = -1,1 do
		for j = -1, 1 do
			if i == 0 or j == 0 then 
				local neighborTile = self.grid[x+i] and self.grid[x+i][y+j]
				if neighborTile and neighborTile.material and tile.enabled  then
					tile.available = true
				end
			end
		end
	end
end

function menu:GetMaxProperty(property)
	local count = 0
	for i = 1, self:GetWidth() do
		for j = 1, self:GetHeight() do
			if self.grid[i][j].picker then
				if self.grid[i][j].picker.description == property then
					count = count + 1
				end
			end
		end
	end
	return count
end

function menu:GetProperty(property)
	local count = 0
	for i = 1, self:GetWidth() do
		for j = 1, self:GetHeight() do
			if self.grid[i][j].picker and self.grid[i][j].material then
				if self.grid[i][j].picker.description == property then
					count = count + 1
				end
			end
		end
	end
	return count
end

function menu:UpdateProperties()

		local maxSharpness = self:GetMaxProperty("sharpness")
		local maxStrength = self:GetMaxProperty("structure")

		local sharpness = self:GetProperty("sharpness")
		local strength = self:GetProperty("structure")	



		local sharpnesses = {}
		tinsert(sharpnesses, "blunt")
		tinsert(sharpnesses, "rusty")
		tinsert(sharpnesses, "dull")
		tinsert(sharpnesses, "honed")
		tinsert(sharpnesses, "sharpened")

		local strengths = {}
		tinsert(strengths, "pathetic")
		tinsert(strengths, "weak")
		tinsert(strengths, "tough")
		tinsert(strengths, "reinforced")

		local sharpVal = floor((sharpness/maxSharpness)*(#sharpnesses-1))
		local strengthVal = floor((strength/maxStrength)*(#strengths-1))
		if strength == 0 then
			menu.weaponName = "hilt"
		elseif sharpness > 0 then
			menu.weaponName = sharpnesses[sharpVal+1].." "..strengths[strengthVal+1].." "..self.weaponType
		else
			menu.weaponName = sharpnesses[sharpVal+1].." "..strengths[strengthVal+1].." ".."club"
		end
end

function menu:UpdateTiles()

	self:TileIterator(ResetTileLinks)
	
	-- set the seed to linked again
	self.grid[self:GetSeedX()][self:GetSeedY()].linked = true

	self:CheckTileLink(self:GetSeedX(),self:GetSeedY())

	--remove any that arent linked
	self:TileIterator(RemoveUnlinkedTile)

	self:TileIterator(UpdateTileAvailability)

	self:UpdateProperties()

end

function menu:GetTileSize()
	return self.template.config.tileSize
end

function menu:PalleteClick(x,y,button)
	if x > self:GetWidth()*self:GetTileSize() + 10 and x < self:GetWidth()*self:GetTileSize() + 28 then
		
		for k,v in pairs(self.pallete.materials) do
			if y > k*20 and y < (k+1)*20 then
				self.currentMaterial = v
				print(v.description)
			end
		end
	end
end

function menu:MousePressed(x,y,button)
	local xgrid = math.floor(x/self:GetTileSize())
	local ygrid = math.floor(y/self:GetTileSize())
	
	local tile = self.grid[xgrid] and self.grid[xgrid][ygrid]
	if tile then
		self:ClickTile(tile)
	end
	self:UpdateTiles()
  self:PalleteClick(x,y,button)
end

function menu:Bake()

end

function menu:KeyPressed(key)
	self:Bake()
end

function menu:DrawPallete()
	for k,v in pairs(self.pallete.materials ) do
		lg.setColor(v.color)
		lg.rectangle("fill", self:GetWidth()*self:GetTileSize() + 10, k*20, 18, 18)
		lg.setColor(255,255,255,255)
		lg.print(v.description, self:GetWidth()*self:GetTileSize() + 30, k*20)
	end
end

function menu:Draw()
	self:TileIterator(DrawTile)
	lg.print("gems: "..self.gems, 200, 30)
	if menu.weaponName then
		lg.print(menu.weaponName)
	end
	self:DrawPallete()
end

function menu:GetPicker(description)
	for k,v in pairs(self.pickers) do
		if v.description == description then
			return v
		end
	end
end


function menu:InitialiseGrid()
	for i = 1, self:GetWidth() do
		self.grid[i] = self.grid[i] or {}
		for j = 1, self:GetHeight() do
			self.grid[i][j] = {}
			if self.template.grid[i] and self.template.grid[i][j] then
				if self.template.grid[i][j] == "seed" then
					
					self.template.seedX = i
					self.template.seedY = j
					self.grid[i][j].picker = self:GetPicker(self.template.grid[i][j])
				end
				self.grid[i][j].picker = self:GetPicker(self.template.grid[i][j])
			else
				self.grid[i][j].disabled = true
			end
		end
	end
end

function menu:LoadPalleteMaterials()
	local pallete = self.pallete
	pallete.x = 10

end

function NewCraftMenu(template, pickers, materials)
	local self = setmetatable({}, menu)
	self.gems = 200
	self.grid = {}
	self.weaponType = "Axe"
	self.template = template
	self.pickers = pickers
	self:InitialiseGrid()
	self.pallete = {}
	self.pallete.materials = materials

	self:LoadPalleteMaterials()

  self.currentMaterial = materials[1]
	self:UpdateTiles()

	return self
end

