local moldgen = {}
moldgen.__index = moldgen
local tinsert = table.insert
local lg = love.graphics
local floor = math.floor



function moldgen:GetPickerOffsetX()
	return self.menu.x + self.menu.offsetX
end

function moldgen:GetPickerOffsetY(i)
	return self.menu.offsetY + i*self.menu.spacing
end
function moldgen:GetPickerSize()
	return self.menu.pickerSize
end

function moldgen:DrawPickers()
	
	for k,v in pairs(self.menu.pickers) do
		lg.setColor(v.color)
		lg.rectangle("fill", self:GetPickerOffsetX(), self:GetPickerOffsetY(k), self:GetPickerSize(),self:GetPickerSize())
		love.graphics.print(v.description, self:GetPickerOffsetX()+self:GetPickerSize()+ 10, self:GetPickerOffsetY(k))
	end
end

function moldgen:AddPicker(description, color)
	local picker = {description = description, color = color}
	tinsert(self.menu.pickers, picker)
end

function moldgen:PickerMousePressed(x,y,button)
	for k,picker in pairs(self.menu.pickers) do
		if (x > self:GetPickerOffsetX()) and (x < self:GetPickerOffsetX()+self:GetPickerSize()) then
			if (y > self:GetPickerOffsetY(k)) and (y < self:GetPickerOffsetY(k)+ self:GetPickerSize()) then
				self.menu.selectedPicker = picker
			end 
		end
	end
end

function moldgen:InitialiseGrid()
	for i = 1, self:GetWidth() do
		self.grid[i] = self.grid[i] or {}
		for j = 1, self:GetHeight() do
			self.grid[i][j] = {}
		end
	end
end

function moldgen:MousePressed(x,y,button)
	local i = floor(x/self:GetTileSize())
	local j = floor(y/self:GetTileSize())
	local tile = self.grid[i] and self.grid[i][j]
	if tile then
		if button == "l" then
			self.grid[i][j] = self.menu.selectedPicker
		elseif button == "r" then
			self.grid[i][j] = {}
		end
	end
	self:PickerMousePressed(x,y,button)
end

function moldgen:GetWidth()
	return self.config.width
end

function moldgen:GetHeight()
	return self.config.height
end

function moldgen:SetWidth(width)
	self.config.width = width
end

function moldgen:SetHeight(height)
	self.config.height = height
end

function moldgen:GetTileSize()
	return self.config.tileSize
end

function moldgen:GetPicker(desc)
	for k,v in pairs(self.menu.pickers) do
		print(v.description, unpack(v.color))
		if v.description == desc then
			return v
		end
	end
	return {description = "none", {255,255,255,255}}
end

function moldgen:Draw()
	local tileSize = self:GetTileSize()
	for i = 1, self:GetWidth() do
		for j = 1, self:GetHeight() do
			lg.setColor(30,30,50,255)
			lg.rectangle("line", i*tileSize, j*tileSize, tileSize, tileSize)
			if self.grid[i][j].description then
				lg.setColor(unpack(self.grid[i][j].color))
				lg.rectangle("fill", i*tileSize, j*tileSize, tileSize, tileSize)
			end
		end
	end
	self:DrawPickers(self:GetWidth()*tileSize)
end

function moldgen:LoadGrid(grid)
	for i = 1, self:GetWidth() do
		for j = 1, self:GetHeight() do
			if grid[i] and grid[i][j] then
				self.grid[i][j] = self:GetPicker(grid[i][j])
			end
		end
	end
end


function moldgen:GetCleanGrid()
	local grid = {}

	for i = 1, self:GetWidth() do
		for j = 1, self:GetHeight() do
			if self.grid[i][j].description then
				grid[i] = grid[i] or {}
				grid[i][j] = self.grid[i][j].description
			end
		end
	end
	return grid
end

function moldgen:KeyPressed(key)
	if key == "up" then
		self:SetWidth(self:GetWidth()+1)
		self:SetHeight(self:GetHeight()+1)
		self:Initialise()
	elseif key =="down" then
		self:SetWidth(self:GetWidth()-1)
		self:SetHeight(self:GetHeight()-1)
		self:Initialise()
	end

	if key == "q" then
		local t = {}
		t.config = self.config
		t.grid = self:GetCleanGrid()

		local s = Tserial.pack(t, nil, true)

		local file = love.filesystem.newFile('testmold.lua')
		file:open('w')
		file:write(s)

	end

	if key == "w" then
		local file = love.filesystem.read('testmold.lua')
		local t = Tserial.unpack(file)
		self:Initialise(t.grid, t.config)
	end
end



function moldgen:Initialise(grid, config)
	self.grid = {}
	self.config = config or self.config
	self:InitialiseGrid()
	if grid then
		self:LoadGrid(grid)
	end
	self.menu.x = self:GetWidth()*self:GetTileSize()
end


function NewMoldGen(pickers)
	local mold = setmetatable({}, moldgen)
	
	mold.config = {}
	mold.config.width = 20
	mold.config.height = 20
	mold.config.tileSize = 20
	mold.grid = {}
	mold:InitialiseGrid()
	

	mold.menu = {}
	mold.menu.offsetX = 25
	mold.menu.offsetY = 20

	mold.menu.pickers = pickers
	mold.menu.spacing = 25
	mold.menu.pickerSize = 20

	mold.menu.x = mold:GetWidth()*mold:GetTileSize()


	mold.menu.selectedPicker = mold.menu.pickers[1]

	return mold
end

return moldgen